@@ -80,7 +80,7 @@ def mproc_apply(func, iterLst, descr='', iterArgPos=0,
 
   if debug:
     def applyFunc(func_, args):
-      return func_(*args)
+      return func_(*args, **extraKwargs)
   else:
     if pool is None:
       pool = mp.Pool()
