@@ -13,6 +13,7 @@ from pyqtgraph.parametertree.parameterTypes import ActionParameterItem, ActionPa
 
 from .prjparam import PrjParam
 from .shortcut import ShortcutsEditor
+from .. import fns
 from ..fns import clsNameOrGroup, params_flattened, popupFilePicker
 from ..widgets import PopupLineEditor, ButtonCollection
 
@@ -26,12 +27,75 @@ class MonkeyPatchedTextParameterItem(TextParameterItem):
 # Monkey patch pyqtgraph text box to allow tab changing focus
 TextParameter.itemClass = MonkeyPatchedTextParameterItem
 
+class ParamDialog(QtWidgets.QDialog):
+  def __init__(self, param: Parameter, parent=None):
+    super().__init__(parent)
+    self.setModal(True)
+    self.param = param
+    self.tree = fns.flexibleParamTree(param)
+    self.saveChanges = False
+
+    layout = QtWidgets.QVBoxLayout()
+    self.setLayout(layout)
+    layout.addWidget(self.tree)
+
+    okBtn = QtWidgets.QPushButton('Ok')
+    cancelBtn = QtWidgets.QPushButton('Cancel')
+    btnLay = QtWidgets.QHBoxLayout()
+    btnLay.addWidget(okBtn)
+    btnLay.addWidget(cancelBtn)
+    layout.addLayout(btnLay)
+
+    def okClicked():
+      self.saveChanges = True
+      self.accept()
+
+    def cancelClicked():
+      self.saveChanges = False
+      self.reject()
+
+    okBtn.clicked.connect(okClicked)
+    cancelBtn.clicked.connect(cancelClicked)
+
+class PgPopupDelegate(QtWidgets.QStyledItemDelegate):
+  """
+  For pyqtgraph-registered parameter types that don't define an itemClass with `makeWidget`, this
+  popup delegate can be used instead which creates a popout parameter tree.
+  """
+  def __init__(self, paramDict: dict, parent=None):
+    super().__init__(parent)
+    paramDict.setdefault('name', paramDict['type'])
+    self.param = Parameter.create(**paramDict)
+
+  def createEditor(self, parent, option, index: QtCore.QModelIndex):
+    self.param.setValue(index.data(QtCore.Qt.ItemDataRole.EditRole))
+    editor = ParamDialog(self.param)
+    editor.show()
+    editor.resize(editor.width() + 50, editor.height() + 30)
+
+    return editor
+
+  def setModelData(self, editor: QtWidgets.QWidget,
+                   model: QtCore.QAbstractTableModel,
+                   index: QtCore.QModelIndex):
+    if editor.saveChanges:
+      model.setData(index, editor.param.value())
+
+  def setEditorData(self, editor: QtWidgets.QWidget, index):
+    value = index.data(QtCore.Qt.ItemDataRole.EditRole)
+    self.param.setValue(value)
+
+  def updateEditorGeometry(self, editor: QtWidgets.QWidget,
+                           option: QtWidgets.QStyleOptionViewItem,
+                           index: QtCore.QModelIndex):
+    return
+
 class PgParamDelegate(QtWidgets.QStyledItemDelegate):
   def __init__(self, paramDict: dict, parent=None):
     super().__init__(parent)
     errMsg = f'{self.__class__} can only create parameter editors from' ' registered pg widgets that implement ' \
              f'makeWidget() and are in pyqtgraph\'s *PARAM_TYPES*.\n' \
-             f'Cannot find {self.__class__}'
+             f'These requirements are not met for type "{paramDict["type"]}"'
 
     if paramDict['type'] not in PARAM_TYPES:
       raise TypeError(errMsg)
