@@ -1,19 +1,16 @@
 from __future__ import annotations
 
 from functools import singledispatch
-from functools import wraps
-from typing import List, Sequence, Callable, Union
+from typing import Sequence, Callable
 
 from pyqtgraph.parametertree import Parameter
-from pyqtgraph.parametertree.Parameter import PARAM_TYPES
 from pyqtgraph.parametertree.parameterTypes import GroupParameter
 
-from .prjparam import PrjParam
 from .. import fns
 from ..misc import CompositionMixin
 from ..processing import *
 
-__all__ = ['NestedProcWrapper']
+__all__ = ['NestedProcWrapper', 'addStageToParam']
 
 def _hookupCondition(parentParam: Parameter, chParam: Parameter):
   condition = chParam.opts.get('condition', None)
@@ -43,50 +40,40 @@ def _hookupCondition(parentParam: Parameter, chParam: Parameter):
 def _mkProcParam(stage: ProcessStage):
   return {'name': stage.name, 'type': 'procgroup', 'process': stage, 'enabled': not stage.disabled}
 
-def atomicRunWrapper(proc: AtomicProcess, param: Parameter):
-  def onChange(p, val):
-    proc.input.update(**{param.name(): val})
-    proc.result = None
-  param.sigValueChanged.connect(onChange)
-
 @singledispatch
 def addStageToParam(stage: ProcessStage, parentParam: Parameter, **kwargs):
   pass
 
 @addStageToParam.register
-def addAtomicToParam(stage: AtomicProcess, parentParam: Parameter,
-                     argNameFormat: Callable[[str], str]=None, nestHyperparams=True, **kwargs):
-  docParams = fns.docParser(stage.fnDoc)
+def addAtomicToParam(stage: AtomicProcess,
+                     parentParam: Parameter,
+                     argNameFormat: Callable[[str], str]=None,
+                     nestHyperparams=True,
+                     **kwargs):
+  descr = fns.docParser(stage.fnDoc).get('func-description', '')
   if nestHyperparams:
     parentParam = fns.getParamChild(parentParam, chOpts=_mkProcParam(stage))
-  if docParams['top-descr']:
-    parentParam.setOpts(tip=docParams['top-descr'])
+  if descr:
+    parentParam.setOpts(tip=descr)
   for key in stage.input.hyperParamKeys:
-    val = stage.input[key]
-    curParam = docParams.get(key, None)
-    if curParam is None:
-      curParam = PrjParam(name=key, value=val)
-    else:
-      if val is not stage.input.FROM_PREV_IO:
-        curParam.value = val
-      if curParam.pType is None:
-        curParam.pType = type(val).__name__.lower()
-    if curParam.opts.get('ignore', False):
-      # Disregard this parameter
-      continue
-    paramDict = curParam.toPgDict()
-    if paramDict['type'] not in PARAM_TYPES:
-      continue
-    if argNameFormat is not None and 'title' not in paramDict:
-      paramDict['title'] = argNameFormat(key)
-    pgParam = fns.getParamChild(parentParam, chOpts=paramDict)
-    atomicRunWrapper(stage, pgParam)
+    pgParam = stage.input.params[key]
+    if key in parentParam.names:
+      # Child already exists, instead of adding this one, reassign the input
+      stage.input.params[key] = parentParam.child(key)
+    elif isinstance(pgParam, Parameter):
+      if argNameFormat is not None:
+        pgParam.setOpts(title=argNameFormat(key))
+      parentParam.addChild(pgParam)
     _hookupCondition(parentParam, pgParam)
-  return stage
+  return parentParam
 
 @addStageToParam.register
-def addNestedToParam(stage: NestedProcess, parentParam: Parameter, nestHyperparams=True,
-                     argNameFormat: Callable[[str], str]=None, treatAsAtomic=False, **kwargs):
+def addNestedToParam(stage: NestedProcess,
+                     parentParam: Parameter,
+                     nestHyperparams=True,
+                     argNameFormat: Callable[[str], str]=None,
+                     treatAsAtomic=False,
+                     **kwargs):
   if nestHyperparams:
     parentParam = fns.getParamChild(parentParam, chOpts=_mkProcParam(stage))
   if treatAsAtomic:
@@ -94,13 +81,9 @@ def addNestedToParam(stage: NestedProcess, parentParam: Parameter, nestHyperpara
     collapsed.input = stage.input
     addAtomicToParam(collapsed, parentParam, argNameFormat)
     return
-  # Special case of a process comprised of just one atomic function
-  # if len(stage.stages) == 1:
-  #   # Hide this level of the process tree
-  #   addStageToParam(stage.stages[0], parentParam, nestHyperparams=False)
-  #   return
   for childStage in stage:
     addStageToParam(childStage, parentParam)
+  return parentParam
 
 class NestedProcWrapper(CompositionMixin):
   def __init__(self, processor: ProcessStage, parentParam: GroupParameter=None,
